@@ -1,15 +1,14 @@
-/**************************************************************************
- Copyright:
-      (C) 2008 - 2012  Alexander Shaduri <ashaduri 'at' gmail.com>
- License: See LICENSE_gsmartcontrol.txt
-***************************************************************************/
+/******************************************************************************
+License: GNU General Public License v3.0 only
+Copyright:
+	(C) 2008 - 2021 Alexander Shaduri <ashaduri@gmail.com>
+******************************************************************************/
 /// \file
 /// \author Alexander Shaduri
 /// \ingroup applib
 /// \weakgroup applib
 /// @{
 
-#include "app_pango_utils.h"  // app_pango_strip_markup()
 #include "gui_utils.h"
 
 
@@ -127,7 +126,7 @@ bool gui_show_text_entry_dialog(const std::string& title, const std::string& mes
 
 		Gtk::Entry input_entry;
 		input_entry.set_activates_default(true);
-
+		input_entry.set_text(default_str);
 
 		Gtk::Box vbox;
 		vbox.set_spacing(12);
