@@ -1,8 +1,8 @@
-/**************************************************************************
- Copyright:
-      (C) 2008 - 2012  Alexander Shaduri <ashaduri 'at' gmail.com>
- License: See LICENSE_gsmartcontrol.txt
-***************************************************************************/
+/******************************************************************************
+License: GNU General Public License v3.0 only
+Copyright:
+	(C) 2008 - 2021 Alexander Shaduri <ashaduri@gmail.com>
+******************************************************************************/
 /// \file
 /// \author Alexander Shaduri
 /// \ingroup applib
@@ -15,11 +15,25 @@
 #include <vector>
 #include <string>
 
-#include "hz/intrusive_ptr.h"
-
 #include "storage_device.h"
-#include "cmdex_sync.h"
-#include "executor_factory.h"
+#include "command_executor.h"
+#include "command_executor_factory.h"
+
+
+enum class StorageDetectorError {
+	NoSmartctlBinary,
+	NoHelperBinary,
+	SmartctlExecutionError,
+	EmptyCommandOutput,
+	UnsupportedCommandVersion,
+	ParseError,
+	StorageDeviceError,
+	ProcReadError,
+	GeneralDetectionErrors,
+	ConfigError,
+	DevOpenError,
+	InvalidCommandLine,
+};
 
 
 
@@ -27,29 +41,22 @@
 class StorageDetector {
 	public:
 
-		/// Constructor
-		StorageDetector()
-		{
-// 			match_patterns_.push_back("/.+/");  // accept anything, the scanner picks up disks only anyway.
-		}
-
-
 		/// Detects a list of drives. Returns detection error message if error occurs.
-		std::string detect(std::vector<StorageDeviceRefPtr>& put_drives_here,
-				ExecutorFactoryRefPtr ex_factory);
+		[[nodiscard]] hz::ExpectedVoid<StorageDetectorError> detect(std::vector<StorageDevicePtr>& drives,
+				const CommandExecutorFactoryPtr& ex_factory);
 
 
 		/// For each drive, fetch basic data and parse it.
 		/// If \c return_first_error is true, the function returns on the first error.
 		/// \return An empty string. Or, if return_first_error is true, the first error that occurs.
-		std::string fetch_basic_data(std::vector<StorageDeviceRefPtr>& drives,
-				ExecutorFactoryRefPtr ex_factory, bool return_first_error = false);
+		[[nodiscard]] hz::ExpectedVoid<StorageDetectorError> fetch_basic_data(std::vector<StorageDevicePtr>& drives,
+				const CommandExecutorFactoryPtr& ex_factory, bool return_first_error = false);
 
 
 		/// Run detect() and fetch_basic_data().
 		/// \return An error if such occurs.
-		std::string detect_and_fetch_basic_data(std::vector<StorageDeviceRefPtr>& put_drives_here,
-				ExecutorFactoryRefPtr ex_factory);
+		[[nodiscard]] hz::ExpectedVoid<StorageDetectorError> detect_and_fetch_basic_data(std::vector<StorageDevicePtr>& put_drives_here,
+				const CommandExecutorFactoryPtr& ex_factory);
 
 
 // 		void add_match_patterns(std::vector<std::string>& patterns)
@@ -59,21 +66,21 @@ class StorageDetector {
 
 
 		/// Add device patterns to drive detection blacklist
-		void add_blacklist_patterns(std::vector<std::string>& patterns)
+		void add_blacklist_patterns(const std::vector<std::string>& patterns)
 		{
 			blacklist_patterns_.insert(blacklist_patterns_.end(), patterns.begin(), patterns.end());
 		}
 
 
 		/// Get all errors produced by fetch_basic_data().
-		const std::vector<std::string>& get_fetch_data_errors() const
+		[[nodiscard]] const std::vector<std::string>& get_fetch_data_errors() const
 		{
 			return fetch_data_errors_;
 		}
 
 
 		/// Get command output for each error in get_fetch_data_errors().
-		const std::vector<std::string>& get_fetch_data_error_outputs() const
+		[[nodiscard]] const std::vector<std::string>& get_fetch_data_error_outputs() const
 		{
 			return fetch_data_error_outputs_;
 		}
