@@ -1,33 +1,36 @@
-/**************************************************************************
- Copyright:
-      (C) 2008 - 2012  Alexander Shaduri <ashaduri 'at' gmail.com>
- License: See LICENSE_gsmartcontrol.txt
-***************************************************************************/
+/******************************************************************************
+License: GNU General Public License v3.0 only
+Copyright:
+	(C) 2008 - 2021 Alexander Shaduri <ashaduri@gmail.com>
+******************************************************************************/
 /// \file
 /// \author Alexander Shaduri
 /// \ingroup applib
 /// \weakgroup applib
 /// @{
 
-#include "hz/hz_config.h"  // CONFIG_*
+#include "build_config.h"
 
-#if defined CONFIG_KERNEL_FAMILY_WINDOWS
-
-
-#include <windows.h>  // CreateFileA(), CloseHandle(), etc...
 #include <glibmm.h>
 #include <set>
 #include <bitset>
 #include <map>
 #include <vector>
+#include <memory>
+
+#ifdef _WIN32
+	#include <windows.h>  // CreateFileA(), CloseHandle(), etc.
+#endif
 
 #include "hz/win32_tools.h"
 #include "hz/string_sprintf.h"
-#include "hz/fs_path.h"
-#include "rconfig/rconfig_mini.h"
-#include "app_pcrecpp.h"
+#include "hz/fs.h"
+#include "hz/string_num.h"
+#include "rconfig/rconfig.h"
+#include "app_regex.h"
 #include "storage_detector_win32.h"
 #include "storage_detector_helpers.h"
+#include "smartctl_executor.h"  // get_smartctl_binary
 
 
 
@@ -103,54 +106,69 @@ std::map<char, DriveLetterInfo> win32_get_drive_letter_map()
 {
 	std::map<char, DriveLetterInfo> drive_letter_map;
 
+#ifdef _WIN32
 	std::bitset<32> drives(GetLogicalDrives());
 
 	// Check which drives are fixed
 	std::vector<char> good_drives;
 	for (char c = 'A'; c <= 'Z'; ++c) {
-		if (drives[c - 'A']) {  // drive is present
+		if (drives[static_cast<unsigned char>(c) - 'A']) {  // drive is present
 			debug_out_dump("app", "Windows drive found: " << c << ".\n");
-			if (GetDriveType((c + std::string(":\\")).c_str()) == DRIVE_FIXED) {
-				debug_out_dump("app", "Windows drive " << c << " is fixed.\n");
-				good_drives.push_back(c);
+			switch (auto drive_type = GetDriveType((c + std::string(":\\")).c_str())) {
+				case DRIVE_FIXED:
+					debug_out_dump("app", "Windows reports the drive " << c << " as fixed.\n");
+					good_drives.push_back(c);
+					break;
+				case DRIVE_REMOVABLE:
+					debug_out_dump("app", "Windows reports the drive " << c << " as removable.\n");
+					good_drives.push_back(c);
+					break;
+				case DRIVE_CDROM:
+					debug_out_dump("app", "Windows reports the drive " << c << " as cdrom.\n");
+					good_drives.push_back(c);
+					break;
+				default:
+					debug_out_dump("app", "Windows reports the drive " << c << " as type " << drive_type << ".\n");
+					// don't add
+					break;
 			}
 		}
 	}
 
 	// Try to open each drive, check its disk extents
-	for (std::size_t i = 0; i < good_drives.size(); ++i) {
-		char drive = good_drives[i];
-		std::string drive_str = std::string("\\\\.\\") + drive + ":";
+	for (const char drive : good_drives) {
+		const std::string drive_str = std::string("\\\\.\\") + drive + ":";
 		HANDLE h = CreateFileA(
-				drive_str.c_str(), GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL,
-				OPEN_EXISTING, FILE_FLAG_NO_BUFFERING | FILE_FLAG_RANDOM_ACCESS, NULL);
+				drive_str.c_str(), GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, nullptr,
+				OPEN_EXISTING, FILE_FLAG_NO_BUFFERING | FILE_FLAG_RANDOM_ACCESS, nullptr);
 		if (h == INVALID_HANDLE_VALUE) {
 			debug_out_warn("app", "Windows drive " << drive << " cannot be opened.\n");
 			continue;
 		}
 		DWORD bytesReturned = 0;
 		VOLUME_DISK_EXTENTS vde;
-		if (!DeviceIoControl(
+		if (DeviceIoControl(
 				h, IOCTL_VOLUME_GET_VOLUME_DISK_EXTENTS,
-				NULL, 0, &vde, sizeof(vde), &bytesReturned, NULL)) {
+				nullptr, 0, &vde, sizeof(vde), &bytesReturned, nullptr) == FALSE) {
 			debug_out_warn("app", "Windows drive " << drive << " is not mapped to any physical drives.\n");
 			continue;
 		}
 
 		std::set<int> physical_drives;
-		for (int i = 0; i < vde.NumberOfDiskExtents; ++i) {
-			physical_drives.insert(vde.Extents[i].DiskNumber);
+		for (std::size_t i = 0; i < vde.NumberOfDiskExtents; ++i) {
+			physical_drives.insert(int(vde.Extents[i].DiskNumber));
 			debug_out_dump("app", "Windows drive " << drive << " corresponds to physical drive " << vde.Extents[i].DiskNumber << ".\n");
 		}
 
 		std::string volume_name;
-		wchar_t volume_name_w[MAX_PATH+1] = {0};
+		std::array<wchar_t, MAX_PATH+1> volume_name_w = {};
 		DWORD dummy = 0;
-		hz::scoped_array<wchar_t> drive_name(hz::win32_utf8_to_utf16((drive + std::string(":\\")).c_str()));
-		if (drive_name && GetVolumeInformationW(drive_name.get(),
-				volume_name_w, MAX_PATH+1,
-				NULL, &dummy, &dummy, NULL, 0)) {
-			volume_name = hz::win32_utf16_to_utf8_string(volume_name_w);
+		const std::wstring drive_name = hz::win32_utf8_to_utf16(drive + std::string(":\\"));
+		if (!drive_name.empty()
+				&& GetVolumeInformationW(drive_name.c_str(),
+					volume_name_w.data(), MAX_PATH+1,
+					nullptr, &dummy, &dummy, nullptr, 0) == TRUE) {
+			volume_name = hz::win32_utf16_to_utf8(volume_name_w);
 		}
 
 		DriveLetterInfo dli;
@@ -159,7 +177,7 @@ std::map<char, DriveLetterInfo> win32_get_drive_letter_map()
 
 		drive_letter_map[drive] = dli;
 	}
-
+#endif
 	return drive_letter_map;
 }
 
@@ -168,46 +186,54 @@ std::map<char, DriveLetterInfo> win32_get_drive_letter_map()
 /// Run "smartctl --scan-open" and pick the devices which have
 /// a port parameter. We don't pick the others because the may
 /// conflict with pd* devices, and we like pd* better than sd*.
-std::string get_scan_open_multiport_devices(std::vector<StorageDeviceRefPtr>& drives,
-		ExecutorFactoryRefPtr ex_factory,
+hz::ExpectedVoid<StorageDetectorError> get_scan_open_multiport_devices(std::vector<StorageDevicePtr>& drives,
+		const CommandExecutorFactoryPtr& ex_factory,
 		const std::map<char, DriveLetterInfo>& drive_letter_map,
 		std::set<int>& equivalent_pds)
 {
 	debug_out_info("app", "Getting multi-port devices through smartctl --scan-open...\n");
 
-	hz::intrusive_ptr<CmdexSync> smartctl_ex = ex_factory->create_executor(ExecutorFactory::ExecutorSmartctl);
+	std::shared_ptr<CommandExecutor> smartctl_ex = ex_factory->create_executor(CommandExecutorFactory::ExecutorType::Smartctl);
 
-	std::string smartctl_binary = get_smartctl_binary();
+	auto smartctl_binary = get_smartctl_binary();
 
 	if (smartctl_binary.empty()) {
 		debug_out_error("app", DBG_FUNC_MSG << "Smartctl binary is not set in config.\n");
-		return "Smartctl binary is not specified in configuration.";
+		return hz::Unexpected(StorageDetectorError::NoSmartctlBinary, _("Smartctl binary is not specified in configuration."));
 	}
 
-	std::string smartctl_def_options;
-	rconfig::get_data("system/smartctl_options", smartctl_def_options);
 
-	if (!smartctl_def_options.empty())
-		smartctl_def_options += " ";
+	auto smartctl_def_options_str = rconfig::get_data<std::string>("system/smartctl_options");
+	std::vector<std::string> smartctl_options;
+	if (!smartctl_def_options_str.empty()) {
+		try {
+			smartctl_options = Glib::shell_parse_argv(smartctl_def_options_str);
+		}
+		catch(Glib::ShellError& e)
+		{
+			return hz::Unexpected(StorageDetectorError::InvalidCommandLine, _("Invalid command line specified."));
+		}
+	}
+	smartctl_options.emplace_back("--scan-open");
+	smartctl_ex->set_command(hz::fs_path_to_string(smartctl_binary), smartctl_options);
 
-	smartctl_ex->set_command(Glib::shell_quote(smartctl_binary),
-			smartctl_def_options + "--scan-open");
-
-	if (!smartctl_ex->execute() || !smartctl_ex->get_error_msg().empty()) {
+	if (const bool execute_status = smartctl_ex->execute(); !execute_status) {
 		debug_out_warn("app", DBG_FUNC_MSG << "Smartctl binary did not execute cleanly.\n");
-		return smartctl_ex->get_error_msg();
+		return hz::Unexpected(StorageDetectorError::SmartctlExecutionError, smartctl_ex->get_error_msg());
 	}
 
 	// any_to_unix is needed for windows
-	std::string output = hz::string_trim_copy(hz::string_any_to_unix_copy(smartctl_ex->get_stdout_str()));
+	const std::string output = hz::string_trim_copy(hz::string_any_to_unix_copy(smartctl_ex->get_stdout_str()));
 	if (output.empty()) {
 		debug_out_error("app", DBG_FUNC_MSG << "Smartctl returned an empty output.\n");
-		return "Smartctl returned an empty output.";
+		return hz::Unexpected(StorageDetectorError::EmptyCommandOutput, _("Smartctl returned an empty output."));
 	}
 
-	// if we've reached smartctl port limit (older versions may have smaller limits), abort.
-	if (app_pcre_match("/UNRECOGNIZED OPTION/mi", output)) {
-		return "Smartctl doesn't support --scan-open switch.";
+	if (app_regex_partial_match("/UNRECOGNIZED OPTION/mi", output)) {
+		// Our requirements list smartctl with --scan-open support, so this should never happen.
+		// Therefore, we don't translate it.
+		return hz::Unexpected(StorageDetectorError::UnsupportedCommandVersion,
+				"Unsupported smartctl version: Smartctl doesn't support --scan-open switch.");
 	}
 
 
@@ -220,27 +246,27 @@ std::string get_scan_open_multiport_devices(std::vector<StorageDeviceRefPtr>& dr
 // /dev/sde,2 -d ata [ATA] (opened)
 
 	// we only pick the ones with ports
-	const pcrecpp::RE port_re = app_pcre_re("/^(/dev/[a-z0-9]+),([0-9]+)[ \\t]+-d[ \\t]+([^ \\t\\n]+)/i");
-	const pcrecpp::RE dev_re = app_pcre_re("/^/dev/sd([a-z])$/");
+	const auto port_re = app_regex_re("/^(/dev/[a-z0-9]+),([0-9]+)[ \\t]+-d[ \\t]+([^ \\t\\n]+)/i");
+	const auto dev_re = app_regex_re("/^/dev/sd([a-z])$/");
 
-	for (std::size_t i = 0; i < lines.size(); ++i) {
+	for (const auto& line : lines) {
 		std::string dev, port_str, type;
-		if (port_re.PartialMatch(hz::string_trim_copy(lines.at(i)), &dev, &port_str, &type)) {
+		if (app_regex_partial_match(port_re, hz::string_trim_copy(line), {&dev, &port_str, &type})) {
 			std::string sd_letter;
 			int drive_num = -1;
-			if (dev_re.PartialMatch(dev, &sd_letter)) {
+			if (app_regex_partial_match(dev_re, dev, &sd_letter)) {
 				// don't use pd* devices equivalent to these sd* devices.
 				drive_num = sd_letter.at(0) - 'a';
 				equivalent_pds.insert(drive_num);
 			}
 
-			std::string full_dev = dev + "," + port_str;
-			StorageDeviceRefPtr drive(new StorageDevice(full_dev, type));
+			const std::string full_dev = dev + "," + port_str;
+			auto drive = std::make_shared<StorageDevice>(full_dev, type);
 
 			std::map<char, std::string> letters_volnames;
-			for (std::map<char, DriveLetterInfo>::const_iterator iter = drive_letter_map.begin(); iter != drive_letter_map.end(); ++iter) {
-				if (iter->second.physical_drives.count(drive_num) > 0) {
-					letters_volnames[iter->first] = iter->second.volume_name;
+			for (const auto& iter : drive_letter_map) {
+				if (iter.second.physical_drives.contains(drive_num)) {
+					letters_volnames[iter.first] = iter.second.volume_name;
 				}
 			}
 			drive->set_drive_letters(letters_volnames);
@@ -249,7 +275,7 @@ std::string get_scan_open_multiport_devices(std::vector<StorageDeviceRefPtr>& dr
 		}
 	}
 
-	return std::string();
+	return {};
 }
 
 
@@ -257,12 +283,12 @@ std::string get_scan_open_multiport_devices(std::vector<StorageDeviceRefPtr>& dr
 
 /// Find and execute areca cli with specified options, return its output through \c output.
 /// \return error message
-inline std::string execute_areca_cli(ExecutorFactoryRefPtr ex_factory, const std::string& cli_binary,
-		const std::string& command_options, std::string& output)
+inline hz::ExpectedVoid<StorageDetectorError> execute_areca_cli(const CommandExecutorFactoryPtr& ex_factory, const std::string& cli_binary,
+		const std::vector<std::string>& command_options, std::string& output)
 {
-	hz::intrusive_ptr<CmdexSync> executor = ex_factory->create_executor(ExecutorFactory::ExecutorArecaCli);
+	std::shared_ptr<CommandExecutor> executor = ex_factory->create_executor(CommandExecutorFactory::ExecutorType::ArecaCli);
 
-	executor->set_command(Glib::shell_quote(cli_binary), command_options);
+	executor->set_command(cli_binary, command_options);
 
 	if (!executor->execute() || !executor->get_error_msg().empty()) {
 		debug_out_warn("app", DBG_FUNC_MSG << "Error while executing Areca cli binary.\n");
@@ -272,10 +298,11 @@ inline std::string execute_areca_cli(ExecutorFactoryRefPtr ex_factory, const std
 	output = hz::string_trim_copy(hz::string_any_to_unix_copy(executor->get_stdout_str()));
 	if (output.empty()) {
 		debug_out_error("app", DBG_FUNC_MSG << "Areca cli returned an empty output.\n");
-		return "Areca CLI returned an empty output.";
+		return hz::Unexpected(StorageDetectorError::EmptyCommandOutput,
+				"Areca CLI returned an empty output.");
 	}
 
-	return std::string();
+	return {};
 }
 
 
@@ -362,8 +389,9 @@ GuiErrMsg<0x00>: Success.
 GuiErrMsg<0x00>: Success.
 ------------------------------------------------------------
 </pre> */
-inline std::string areca_cli_get_drives(const std::string& cli_binary, const std::string& dev, int controller,
-		std::vector<StorageDeviceRefPtr>& drives, ExecutorFactoryRefPtr ex_factory)
+[[nodiscard]] inline hz::ExpectedVoid<StorageDetectorError> areca_cli_get_drives(
+		const std::string& cli_binary, const std::string& dev, int controller,
+		std::vector<StorageDevicePtr>& drives, const CommandExecutorFactoryPtr& ex_factory)
 {
 	debug_out_info("app", "Getting available Areca drives (ports) for controller " << controller << " through Areca CLI...\n");
 
@@ -372,84 +400,85 @@ inline std::string areca_cli_get_drives(const std::string& cli_binary, const std
 	// the interactive mode.
 
 	std::string output;
-	std::string error = execute_areca_cli(ex_factory, cli_binary, "disk info", output);
-	if (!error.empty()) {
-		return error;
+	auto execute_status = execute_areca_cli(ex_factory, cli_binary, {"disk", "info"}, output);
+	if (!execute_status) {
+		return execute_status;
 	}
 
 	// split to lines
 	std::vector<std::string> lines;
 	hz::string_split(output, '\n', lines, true);
 
-	enum FormatType {
-		FormatUnknown,
-		FormatNoEnc1,
-		FormatNoEnc2,
-		FormatEnc
+	enum class FormatType {
+		Unknown,
+		NoEnc1,
+		NoEnc2,
+		Enc
 	};
 
-	pcrecpp::RE noenc1_header_re = app_pcre_re("/^\\s*#\\s+Ch#/mi");
-	pcrecpp::RE noenc2_header_re = app_pcre_re("/^\\s*#\\s+ModelName/mi");
-	pcrecpp::RE exp_header_re = app_pcre_re("/^\\s*#\\s+Enc#/mi");
+	const auto noenc1_header_re = app_regex_re("/^\\s*#\\s+Ch#/mi");
+	const auto noenc2_header_re = app_regex_re("/^\\s*#\\s+ModelName/mi");
+	const auto exp_header_re = app_regex_re("/^\\s*#\\s+Enc#/mi");
 
-	FormatType format_type = FormatUnknown;
-	for (std::size_t i = 0; i < lines.size(); ++i) {
-		if (noenc1_header_re.PartialMatch(lines.at(i))) {
-			format_type = FormatNoEnc1;
+	FormatType format_type = FormatType::Unknown;
+	for (const auto& line : lines) {
+		if (app_regex_partial_match(noenc1_header_re, line)) {
+			format_type = FormatType::NoEnc1;
 			break;
-		} else if (noenc2_header_re.PartialMatch(lines.at(i))) {
-			format_type = FormatNoEnc2;
+		}
+		if (app_regex_partial_match(noenc2_header_re, line)) {
+			format_type = FormatType::NoEnc2;
 			break;
-		} else if (exp_header_re.PartialMatch(lines.at(i))) {
-			format_type = FormatEnc;
+		}
+		if (app_regex_partial_match(exp_header_re, line)) {
+			format_type = FormatType::Enc;
 			break;
 		}
 	}
-	if (format_type == FormatUnknown) {
+	if (format_type == FormatType::Unknown) {
 		debug_out_warn("app", "Could not read Areca CLI output: No valid header found.\n");
-		return "Could not read Areca CLI output: No valid header found.";
+		return hz::Unexpected(StorageDetectorError::ParseError,
+				_("Could not read Areca CLI output: No valid header found."));
 	}
 
 	// Note: These may not match the full model, but just the first part is sufficient for comparison with "N.A.".
-	pcrecpp::RE noexp1_port_re = app_pcre_re("/^\\s*[0-9]+\\s+([0-9]+)\\s+([^\\s]+)/mi");  // matches port, model.
-	pcrecpp::RE noexp2_port_re = app_pcre_re("/^\\s*([0-9]+)\\s+([^\\s]+)/mi");  // matches port, model.
-	pcrecpp::RE exp_port_re = app_pcre_re("/^\\s*[0-9]+\\s+([0-9]+)\\s+(?:Slot#|SLOT\\s+)([0-9]+)\\s+([^\\s]+)/mi");  // matches enclosure, port, model.
+	const auto noexp1_port_re = app_regex_re("/^\\s*[0-9]+\\s+([0-9]+)\\s+([^\\s]+)/mi");  // matches port, model.
+	const auto noexp2_port_re = app_regex_re("/^\\s*([0-9]+)\\s+([^\\s]+)/mi");  // matches port, model.
+	const auto exp_port_re = app_regex_re("/^\\s*[0-9]+\\s+([0-9]+)\\s+(?:Slot#|SLOT\\s+)([0-9]+)\\s+([^\\s]+)/mi");  // matches enclosure, port, model.
 
-	bool has_enclosure = (format_type == FormatEnc);
+	const bool has_enclosure = (format_type == FormatType::Enc);
 	if (has_enclosure) {
 		debug_out_dump("app", "Areca controller seems to have enclosures.\n");
 	} else {
 		debug_out_dump("app", "Areca controller doesn't have any enclosures.\n");
 	}
 
-	for (std::size_t i = 0; i < lines.size(); ++i) {
+	for (const auto& line : lines) {
 		std::string port_str, model_str;
 		if (has_enclosure) {
 			std::string enclosure_str;
-			if (exp_port_re.PartialMatch(hz::string_trim_copy(lines.at(i)), &enclosure_str, &port_str, &model_str)) {
+			if (app_regex_partial_match(exp_port_re, hz::string_trim_copy(line), {&enclosure_str, &port_str, &model_str})) {
 				if (model_str != "N.A.") {
-					int port = -1, enclosure = -1;
-					hz::string_is_numeric(port_str, port);
-					hz::string_is_numeric(enclosure_str, enclosure);
-					drives.push_back(StorageDeviceRefPtr(new StorageDevice(dev,
-							"areca," + hz::number_to_string(port) + "/" + hz::number_to_string(enclosure))));
+					const int port = hz::string_to_number_nolocale<int>(port_str);
+					const int enclosure = hz::string_to_number_nolocale<int>(enclosure_str);
+					drives.emplace_back(std::make_shared<StorageDevice>(dev,
+							"areca," + hz::number_to_string_nolocale(port) + "/" + hz::number_to_string_nolocale(enclosure)));
 					debug_out_info("app", "Added Areca drive " << drives.back()->get_device_with_type() << ".\n");
 				}
 			}
 		} else {  // no enclosures
-			pcrecpp::RE port_re = (format_type == FormatNoEnc1 ? noexp1_port_re : noexp2_port_re);
-			if (port_re.PartialMatch(hz::string_trim_copy(lines.at(i)), &port_str, &model_str)) {
+			const auto port_re = (format_type == FormatType::NoEnc1 ? noexp1_port_re : noexp2_port_re);
+			if (app_regex_partial_match(port_re, hz::string_trim_copy(line), {&port_str, &model_str})) {
 				if (model_str != "N.A.") {
-					int port = -1;
-					hz::string_is_numeric(port_str, port);
-					drives.push_back(StorageDeviceRefPtr(new StorageDevice(dev, "areca," + hz::number_to_string(port))));
+					const int port = hz::string_to_number_nolocale<int>(port_str);
+					drives.emplace_back(std::make_shared<StorageDevice>(dev, "areca," + hz::number_to_string_nolocale(port)));
 					debug_out_info("app", "Added Areca drive " << drives.back()->get_device_with_type() << ".\n");
 				}
 			}
 		}
 	}
 
-	return std::string();
+	return {};
 }
 
 
@@ -487,37 +516,40 @@ If CLI is not installed, do the brute-force way:
 		-d areca,[1-128]/[1-8] /dev/arcmsrN
 			It's 2-3 drives a second on an empty port, so some limits are set in config.
 </pre> */
-inline std::string detect_drives_win32_areca(std::vector<StorageDeviceRefPtr>& drives, ExecutorFactoryRefPtr ex_factory)
+inline hz::ExpectedVoid<StorageDetectorError> detect_drives_win32_areca(std::vector<StorageDevicePtr>& drives, const CommandExecutorFactoryPtr& ex_factory)
 {
 	debug_out_info("app", DBG_FUNC_MSG << "Detecting drives behind Areca controller(s)...\n");
 
-	int scan_controllers = rconfig::get_data<int32_t>("system/win32_areca_scan_controllers");
+	const int scan_controllers = rconfig::get_data<int>("system/win32_areca_scan_controllers");
 	if (scan_controllers == 0) {  // disabled
 		debug_out_info("app", "Areca controller scanning is disabled through config.\n");
-		return std::string();
+		return {};
 	}
 
 	// Check if Areca tools are installed
 
 	std::string cli_inst_path;
+#ifdef _WIN32
 	hz::win32_get_registry_value_string(HKEY_LOCAL_MACHINE,
 			"SOFTWARE\\Wow6432Node\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\CLI", "InstallPath", cli_inst_path);
 	if (cli_inst_path.empty()) {
 		hz::win32_get_registry_value_string(HKEY_LOCAL_MACHINE,
 				"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\CLI", "InstallPath", cli_inst_path);
 	}
-
+#endif
 	if (scan_controllers == 2) {  // auto
 		std::string http_inst_path;
+#ifdef _WIN32
 		hz::win32_get_registry_value_string(HKEY_LOCAL_MACHINE,
 				"SOFTWARE\\Wow6432Node\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\archttp", "InstallPath", http_inst_path);
 		if (http_inst_path.empty()) {
 			hz::win32_get_registry_value_string(HKEY_LOCAL_MACHINE,
 					"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\archttp", "InstallPath", http_inst_path);
 		}
+#endif
 		if (cli_inst_path.empty() && http_inst_path.empty()) {
 			debug_out_info("app", "No Areca software found. Install Areca CLI or set \"system/win32_areca_scan_controllers\" config key to 1 to force scanning.\n");
-			return std::string();
+			return {};
 		}
 		if (http_inst_path.empty()) {
 			debug_out_dump("app", "Areca HTTP installation not found.\n");
@@ -531,54 +563,55 @@ inline std::string detect_drives_win32_areca(std::vector<StorageDeviceRefPtr>& d
 		debug_out_dump("app", "Areca CLI installation found at: \"" << cli_inst_path << "\".\n");
 	}
 
-	bool cli_found = !cli_inst_path.empty();
+	const bool cli_found = !cli_inst_path.empty();
 
-	int use_cli = rconfig::get_data<int32_t>("system/win32_areca_use_cli");
+	int use_cli = rconfig::get_data<int>("system/win32_areca_use_cli");
 	bool scan_detect = (use_cli != 1);  // Whether to detect using sequential port scanning. Only do that if CLI is not forced.
 	if (use_cli == 2) {  // auto
-		use_cli = cli_found;
+		use_cli = cli_found ? 1 : 0;
 	}
 
-	std::string cli_binary;
+	hz::fs::path cli_binary;
 	if (use_cli == 1) {
-		cli_binary = rconfig::get_data<std::string>("system/areca_cli_binary");
-		if (!hz::FsPath(cli_binary).is_absolute() && !cli_inst_path.empty()) {
-			cli_binary = hz::FsPath(cli_inst_path).append(cli_binary).str();
+		cli_binary = hz::fs_path_from_string(rconfig::get_data<std::string>("system/areca_cli_binary"));
+		if (!cli_binary.is_absolute() && !cli_inst_path.empty()) {
+			cli_binary = hz::fs_path_from_string(cli_inst_path) / cli_binary;
 		}
-		if (hz::FsPath(cli_binary).is_absolute() && !hz::FsPath(cli_binary).exists()) {
+		if (cli_binary.is_absolute() && !hz::fs::exists(cli_binary)) {
 			use_cli = 0;
-			debug_out_dump("app", "Areca CLI binary \"" << cli_binary << "\" not found.\n");
+			debug_out_dump("app", "Areca CLI binary \"" << cli_binary.string() << "\" not found.\n");
 		}
 	}
 
-	hz::intrusive_ptr<CmdexSync> smartctl_ex = ex_factory->create_executor(ExecutorFactory::ExecutorSmartctl);
+	std::shared_ptr<CommandExecutor> smartctl_ex = ex_factory->create_executor(CommandExecutorFactory::ExecutorType::Smartctl);
 
 	// --- CLI mode
 
 	// Since CLI may segfault if there are no drives, test the controller presence first.
 	// It doesn't matter if we use areca,N or areca,N/E - we will still get a different
 	// error if there's no controller.
-	if (use_cli) {
+	if (use_cli != 0) {
 		debug_out_dump("app", "Testing Areca controller presence using smartctl...\n");
 
-		StorageDeviceRefPtr drive(new StorageDevice("/dev/arcmsr0", "areca,1"));
-		std::string error_msg = drive->fetch_basic_data_and_parse(smartctl_ex);
-		std::string output = drive->get_info_output();
-		if (app_pcre_match("/No Areca controller found/mi", output)
-				|| app_pcre_match("/Smartctl open device: .* failed: No such device/mi", output) ) {
+		auto drive = std::make_shared<StorageDevice>("/dev/arcmsr0", "areca,1");
+		[[maybe_unused]] auto drive_status = drive->fetch_basic_data_and_parse(smartctl_ex);
+		const std::string output = drive->get_basic_output();
+		if (app_regex_partial_match("/No Areca controller found/mi", output)
+				|| app_regex_partial_match("/Smartctl open device: .* failed: No such device/mi", output) ) {
 			use_cli = 0;
 			scan_detect = false;
 			debug_out_dump("app", "Areca controller not found.\n");
 		}
 	}
 
-	if (use_cli) {
+	if (use_cli != 0) {
 		debug_out_info("app", "Scanning Areca drives using CLI...\n");
-		int cli_max_controllers = 1;  // TODO controller # with CLI.
+		const int cli_max_controllers = 1;  // TODO controller # with CLI.
 		for (int controller_no = 0; controller_no < cli_max_controllers; ++controller_no) {
-			std::string error_msg = areca_cli_get_drives(cli_binary, "/dev/arcmsr" + hz::number_to_string(controller_no), controller_no, drives, ex_factory);
+			auto execute_status = areca_cli_get_drives(cli_binary.string(),
+					"/dev/arcmsr" + hz::number_to_string_nolocale(controller_no), controller_no, drives, ex_factory);
 			// If we get an error on controller 0, fall back to no-cli detection.
-			if (!error_msg.empty() && controller_no == 0) {
+			if (!execute_status && controller_no == 0) {
 				use_cli = 0;
 				debug_out_warn("app", "Areca scan using CLI failed.\n");
 				if (scan_detect) {
@@ -592,27 +625,27 @@ inline std::string detect_drives_win32_areca(std::vector<StorageDeviceRefPtr>& d
 	if (use_cli == 0 && scan_detect) {
 		debug_out_info("app", "Manually scanning Areca controllers and ports...\n");
 
-		int max_controllers = rconfig::get_data<int32_t>("system/win32_areca_max_controllers");
-		int max_noenc_ports = rconfig::get_data<int32_t>("system/win32_areca_neonc_max_scan_port");
+		const int max_controllers = rconfig::get_data<int>("system/win32_areca_max_controllers");
+		int max_noenc_ports = rconfig::get_data<int>("system/win32_areca_neonc_max_scan_port");
 		max_noenc_ports = std::max(1, std::min(24, max_noenc_ports));  // 1-24 sanity check
-		int max_enc_ports = rconfig::get_data<int32_t>("system/win32_areca_enc_max_scan_port");
+		int max_enc_ports = rconfig::get_data<int>("system/win32_areca_enc_max_scan_port");
 		max_enc_ports = std::max(1, std::min(128, max_enc_ports));  // 1-128 sanity check
-		int max_enclosures = rconfig::get_data<int32_t>("system/win32_areca_enc_max_enclosure");
+		int max_enclosures = rconfig::get_data<int>("system/win32_areca_enc_max_enclosure");
 		max_enclosures = std::max(1, std::min(8, max_enclosures));  // 1-8 sanity check
 
 		for (int controller_no = 0; controller_no < max_controllers; ++controller_no) {
-			std::string dev = std::string("/dev/arcmsr") + hz::number_to_string(controller_no);
+			const std::string dev = std::string("/dev/arcmsr") + hz::number_to_string_nolocale(controller_no);
 
 			// First, scan using the areca,N format.
 			debug_out_dump("app", "Starting brute-force port scan (no-enclosure) on 1-" << max_noenc_ports << " ports, device \"" << dev
 					<< "\". Change the maximum by setting \"system/win32_areca_neonc_max_scan_port\" config key.\n");
 
-			std::size_t old_drive_count = drives.size();
+			const std::size_t old_drive_count = drives.size();
 			std::string last_output;
-			std::string error_msg = smartctl_scan_drives_sequentially(dev, "areca,%d", 1, max_noenc_ports, drives, ex_factory, last_output);
+			auto scan_status = smartctl_scan_drives_sequentially(dev, "areca,%d", 1, max_noenc_ports, drives, ex_factory, last_output);
 			// If the scan stopped because of no controller, stop it all.
-			if (!error_msg.empty() && (app_pcre_match("/No Areca controller found/mi", last_output)
-					|| app_pcre_match("/Smartctl open device: .* failed: No such device/mi", last_output)) ) {
+			if (!scan_status && (app_regex_partial_match("/No Areca controller found/mi", last_output)
+					|| app_regex_partial_match("/Smartctl open device: .* failed: No such device/mi", last_output)) ) {
 				debug_out_dump("app", "Areca controller " << controller_no << " not present, stopping sequential scan.\n");
 				break;
 			}
@@ -623,7 +656,8 @@ inline std::string detect_drives_win32_areca(std::vector<StorageDeviceRefPtr>& d
 				for (int enclosure_no = 1; enclosure_no < max_enclosures; ++enclosure_no) {
 					debug_out_dump("app", "Starting brute-force port scan (enclosure #" << enclosure_no << ") on 1-" << max_enc_ports << " ports, device \"" << dev
 							<< "\". Change the maximums by setting \"system/win32_areca_onc_max_scan_port\" and \"system/win32_areca_enc_max_enclosure\" config keys.\n");
-					error_msg = smartctl_scan_drives_sequentially(dev, "areca,%d/" + hz::number_to_string(enclosure_no), 1, max_enc_ports, drives, ex_factory, last_output);
+					// FIXME Not sure whether we should ignore this error message
+					[[maybe_unused]] auto encl_status = smartctl_scan_drives_sequentially(dev, "areca,%d/" + hz::number_to_string_nolocale(enclosure_no), 1, max_enc_ports, drives, ex_factory, last_output);
 				}
 			}
 
@@ -633,7 +667,7 @@ inline std::string detect_drives_win32_areca(std::vector<StorageDeviceRefPtr>& d
 
 	debug_out_info("app", DBG_FUNC_MSG << "Finished scanning Areca controllers.\n");
 
-	return std::string();
+	return {};
 }
 
 
@@ -647,47 +681,47 @@ inline std::string detect_drives_win32_areca(std::vector<StorageDeviceRefPtr>& d
 // (or /dev/pdN, /dev/ being optional) where N comes from
 // "\\.\PhysicalDriveN" (winnt only).
 // http://msdn.microsoft.com/en-us/library/aa365247(VS.85).aspx
-std::string detect_drives_win32(std::vector<StorageDeviceRefPtr>& drives, ExecutorFactoryRefPtr ex_factory)
+hz::ExpectedVoid<StorageDetectorError> detect_drives_win32(std::vector<StorageDevicePtr>& drives,
+		const CommandExecutorFactoryPtr& ex_factory)
 {
 	std::vector<std::string> error_msgs;
-	std::string error_msg;
 
 	// Construct drive letter map
 	debug_out_info("app", "Checking which drive corresponds to which \\\\.\\PhysicalDriveN device...\n");
-	std::map<char, DriveLetterInfo> drive_letter_map = win32_get_drive_letter_map();
+	const std::map<char, DriveLetterInfo> drive_letter_map = win32_get_drive_letter_map();
 
 
-	hz::intrusive_ptr<CmdexSync> smartctl_ex = ex_factory->create_executor(ExecutorFactory::ExecutorSmartctl);
+	std::shared_ptr<CommandExecutor> smartctl_ex = ex_factory->create_executor(CommandExecutorFactory::ExecutorType::Smartctl);
 
 	// Fetch multiport devices using --scan-open.
 	// Note that this may return duplicates (e.g. /dev/sda and /dev/csmi0,0)
 
 	std::set<int> used_pds;
-	error_msg = get_scan_open_multiport_devices(drives, ex_factory, drive_letter_map, used_pds);
-	if (!error_msg.empty()) {
-		error_msgs.push_back(error_msg);
+	auto multiport_status = get_scan_open_multiport_devices(drives, ex_factory, drive_letter_map, used_pds);
+	if (!multiport_status) {
+		error_msgs.push_back(multiport_status.error().message());
 	}
 
-	bool multiport_found = !drives.empty();
+	const bool multiport_found = !drives.empty();
 	bool areca_open_found = false;  // whether areca devices were found at --scan-open time.
 
 	// Find out their serial numbers and whether there are Arecas there.
-	std::map<std::string, StorageDeviceRefPtr> serials;
-	for (std::size_t i = 0; i < drives.size(); ++i) {
-		std::string local_error = drives.at(i)->fetch_basic_data_and_parse(smartctl_ex);
-		if (!local_error.empty()) {
-			debug_out_info("app", "Smartctl returned with an error: " << local_error << "\n");
+	std::map<std::string, StorageDevicePtr> serials;
+	for (auto& drive : drives) {
+		const auto local_status = drive->fetch_basic_data_and_parse(smartctl_ex);
+		if (!local_status) {
+			debug_out_info("app", "Smartctl returned with an error: " << local_status.error().message() << "\n");
 			// Don't exit, just report it.
 		}
-		if (!drives.at(i)->get_serial_number().empty()) {
+		if (!drive->get_serial_number().empty()) {
 			// add model as well, who knows, there may be duplicates across vendors
-			std::string drive_serial_id = drives.at(i)->get_model_name() + "_" + drives.at(i)->get_serial_number();
-			serials[drive_serial_id] = drives.at(i);
+			const std::string drive_serial_id = drive->get_model_name() + "_" + drive->get_serial_number();
+			serials[drive_serial_id] = drive;
 		}
 
 		// See if there are any areca devices. This is not implemented yet by smartctl (as of 6.0),
 		// but if it ever is, we can skip our own detection below.
-		std::string type_arg = drives.at(i)->get_type_argument();
+		const std::string type_arg = drive->get_type_argument();
 		if (type_arg.find("areca") != std::string::npos) {
 			areca_open_found = true;
 		}
@@ -698,22 +732,24 @@ std::string detect_drives_win32(std::vector<StorageDeviceRefPtr>& drives, Execut
 
 	debug_out_info("app", "Starting sequential scan of \\\\.\\PhysicalDriveN devices...\n");
 
-	int num_failed = 0;
-	for (int drive_num = 0; ; ++drive_num) {
+	[[maybe_unused]] int num_failed = 0;
+	const int max_drives = 255;  // arbitrary
+	for (int drive_num = 0; drive_num < max_drives; ++drive_num) {
 
 		// If the drive was already encountered in --scan-open (with a port number), skip it.
-		if (used_pds.count(drive_num) > 0) {
+		if (used_pds.contains(drive_num)) {
 			debug_out_dump("app", "pd" << drive_num << " already encountered in --scan-open output (as sd*), skipping.\n");
 			continue;
 		}
 
-		std::string phys_name = hz::string_sprintf("\\\\.\\PhysicalDrive%d", drive_num);
+		const std::string phys_name = hz::string_sprintf("\\\\.\\PhysicalDrive%d", drive_num);
 
+#ifdef _WIN32
 		// If the drive is openable, then it's there. Yes, CreateFile() is open, not create.
 		// NOTE: Administrative privileges are required to open it.
 		// We don't use any long/unopenable files here, so use the ANSI version.
 		HANDLE h = CreateFileA(phys_name.c_str(), 0, FILE_SHARE_READ | FILE_SHARE_WRITE,
-				NULL, OPEN_EXISTING, 0, NULL);
+				nullptr, OPEN_EXISTING, 0, nullptr);
 
 		// The numbers are usually consecutive, but sometimes there are holes when
 		// removable devices are removed. Try 3 extra drives just in case.
@@ -723,39 +759,40 @@ std::string detect_drives_win32(std::vector<StorageDeviceRefPtr>& drives, Execut
 			if (num_failed >= 3) {
 				debug_out_dump("app", "Stopping sequential scan.\n");
 				break;
-			} else {
-				continue;
 			}
+			continue;
 		}
 		CloseHandle(h);
+#endif
 
 		debug_out_dump("app", "Successfully opened \"" << phys_name << "\".\n");
 
-		StorageDeviceRefPtr drive(new StorageDevice(hz::string_sprintf("pd%d", drive_num)));
+		auto drive = std::make_shared<StorageDevice>(hz::string_sprintf("pd%d", drive_num));
 
 		std::map<char, std::string> letters_volnames;
-		for (std::map<char, DriveLetterInfo>::const_iterator iter = drive_letter_map.begin(); iter != drive_letter_map.end(); ++iter) {
-			if (iter->second.physical_drives.count(drive_num) > 0) {
-				letters_volnames[iter->first] = iter->second.volume_name;
+		for (const auto& iter : drive_letter_map) {
+			if (iter.second.physical_drives.contains(drive_num)) {
+				letters_volnames[iter.first] = iter.second.volume_name;
 			}
 		}
 		drive->set_drive_letters(letters_volnames);
 		debug_out_dump("app", "Drive letters for: " << drive->get_device() << ": " << drive->format_drive_letters(true) << ".\n");
 
+		// Fetch the drive data
+		auto local_status = drive->fetch_basic_data_and_parse(smartctl_ex);
+		if (!local_status) {
+			debug_out_info("app", "Smartctl returned with an error: " << local_status.error().message() << "\n");
+			// Don't exit, just report it.
+		}
+
 		// Sometimes, a single physical drive may be accessible from both "/.//PhysicalDriveN"
 		// and "/.//Scsi2" (e.g. pd0 and csmi2,1). Prefer the port-having ones (which is from --scan-open),
 		// they contain more information.
 		// The only way to detect these duplicates is to compare them using serial numbers.
 		if (!serials.empty()) {
-			std::string local_error = drive->fetch_basic_data_and_parse(smartctl_ex);
-			if (!local_error.empty()) {
-				debug_out_info("app", "Smartctl returned with an error: " << local_error << "\n");
-				// Don't exit, just report it.
-			}
-
-			std::string drive_serial_id = drive->get_model_name() + "_" + drive->get_serial_number();
+			const std::string drive_serial_id = drive->get_model_name() + "_" + drive->get_serial_number();
 			// A serial may be empty if "-q noserial" was given to smartctl.
-			if (!drive->get_serial_number().empty() && serials.count(drive_serial_id) > 0) {
+			if (!drive->get_serial_number().empty() && serials.contains(drive_serial_id)) {
 				debug_out_info("app", "Skipping drive due to duplicate S/N: model: \"" << drive->get_model_name()
 						<< "\", S/N: \"" << drive->get_serial_number() << "\".\n");
 				// Copy the drive letters over to previously detected one (since we can't detect drive letters there).
@@ -777,16 +814,17 @@ std::string detect_drives_win32(std::vector<StorageDeviceRefPtr>& drives, Execut
 	if (!multiport_found) {
 		debug_out_info("app", "Checking for additional 3ware devices...\n");
 		std::string inst_path;
+#ifdef _WIN32
 		hz::win32_get_registry_value_string(HKEY_USERS, ".DEFAULT\\Software\\3ware\\3DM2", "InstallPath", inst_path);
-
+#endif
 		if (!inst_path.empty()) {
 			debug_out_dump("app", "3ware 3DM2 found at\"" << inst_path << "\".\n");
 			std::vector<int> controllers;
-			error_msg = tw_cli_get_controllers(ex_factory, controllers);
+			auto tw_status = tw_cli_get_controllers(ex_factory, controllers);
 			// ignore the error message above, it's of no use.
-			for (std::size_t i = 0; i < controllers.size(); ++i) {
+			for (const int controller : controllers) {
 				// don't specify device, it's ignored in tw_cli mode
-				tw_cli_get_drives("", controllers.at(i), drives, ex_factory, true);
+				[[maybe_unused]] auto tw_drive_status = tw_cli_get_drives("", controller, drives, ex_factory, true);
 			}
 		} else {
 			debug_out_info("app", "3ware 3DM2 not installed.\n");
@@ -795,18 +833,20 @@ std::string detect_drives_win32(std::vector<StorageDeviceRefPtr>& drives, Execut
 
 
 	if (!areca_open_found) {
-		detect_drives_win32_areca(drives, ex_factory);
-		if (!error_msg.empty()) {
-			error_msgs.push_back(error_msg);
+		auto areca_status = detect_drives_win32_areca(drives, ex_factory);
+		if (!areca_status) {
+			error_msgs.push_back(areca_status.error().message());
 		}
 	}
 
-	return hz::string_join(error_msgs, "\n");
+	if (!error_msgs.empty()) {
+		return hz::Unexpected(StorageDetectorError::GeneralDetectionErrors, hz::string_join(error_msgs, "\n"));
+	}
+
+	return {};
 }
 
 
 
 
-#endif  // CONFIG_KERNEL_FAMILY_WINDOWS
-
 /// @}
