@@ -14,13 +14,13 @@ Copyright:
 #include <glibmm.h>
 #include <cctype>
 #include <cstdint>
-#include <format>
 #include <memory>
 #include <unordered_map>
 #include <utility>
 #include <string>
 #include <vector>
 
+#include "fmt/format.h"
 #include "rconfig/rconfig.h"
 #include "hz/string_algo.h"  // string_trim_copy, string_any_to_unix_copy
 #include "hz/fs.h"
@@ -155,8 +155,17 @@ hz::ExpectedVoid<StorageDeviceError> StorageDevice::parse_basic_data()
 	// Clear everything fetched before, except outputs and type
 	this->clear_parse_results();
 
+	// Detect the output format.
+	auto detect_status = SmartctlParser::detect_output_format(this->get_basic_output());
+	auto output_format = SmartctlOutputFormat::Text;
+	if (detect_status.has_value()) {
+		output_format = detect_status.value();
+	} else {
+		debug_out_warn("app", "Cannot detect smartctl output format. Assuming Text.\n");
+	}
+
 	// Parse using Basic parser. This supports all drive types.
-	auto basic_parser = SmartctlParser::create(SmartctlParserType::Basic, SmartctlOutputFormat::Json);
+	auto basic_parser = SmartctlParser::create(SmartctlParserType::Basic, output_format);
 	DBG_ASSERT_RETURN(basic_parser, hz::Unexpected(StorageDeviceError::ParseError, _("Cannot create parser")));
 
 	// This also fills the drive type properties.
@@ -164,7 +173,7 @@ hz::ExpectedVoid<StorageDeviceError> StorageDevice::parse_basic_data()
 	if (!parse_status) {
 		std::string message = parse_status.error().message();
 		return hz::Unexpected(StorageDeviceError::ParseError,
-				std::vformat(_("Cannot parse smartctl output: {}"), std::make_format_args(message)));
+				fmt::format(fmt::runtime(_("Cannot parse smartctl output: {}")), message));
 	}
 
 	// See if we can narrow down the drive type from what was detected
@@ -189,7 +198,7 @@ hz::ExpectedVoid<StorageDeviceError> StorageDevice::parse_basic_data()
 	// Note that this may try to parse data the second time (it may already have
 	// been parsed by parse_data() which failed at it).
 //	if (do_set_properties) {
-//		auto parser = SmartctlParser::create(SmartctlParserType::Ata, SmartctlOutputFormat::Json);
+//		auto parser = SmartctlParser::create(SmartctlParserType::Ata, output_format);
 //		DBG_ASSERT_RETURN(parser, hz::Unexpected(StorageDeviceError::ParseError, _("Cannot create parser")));
 //
 //		if (parser->parse(this->basic_output_)) {  // try to parse it
@@ -375,7 +384,7 @@ hz::ExpectedVoid<StorageDeviceError> StorageDevice::try_parse_data()
 		// return full parser's error messages - they are more detailed.
 		std::string message = basic_parse_status.error().message();
 		return hz::Unexpected(StorageDeviceError::ParseError,
-				std::vformat(_("Cannot parse smartctl output: {}"), std::make_format_args(message)));
+				fmt::format(fmt::runtime(_("Cannot parse smartctl output: {}")), message));
 	}
 
 	return {};  // return ok if at least the info was ok.
@@ -412,7 +421,7 @@ hz::ExpectedVoid<StorageDeviceError> StorageDevice::parse_full_data(SmartctlPars
 
 	std::string message = parse_status.error().message();
 	return hz::Unexpected(StorageDeviceError::ParseError,
-			std::vformat(_("Cannot parse smartctl output: {}"), std::make_format_args(message)));
+			fmt::format(fmt::runtime(_("Cannot parse smartctl output: {}")), message));
 }
 
 
@@ -437,7 +446,7 @@ hz::ExpectedVoid<StorageDeviceError> StorageDevice::parse_any_data_for_virtual()
 	if (!basic_parse_status) {
 		std::string message = basic_parse_status.error().message();
 		return hz::Unexpected(StorageDeviceError::ParseError,
-				std::vformat(_("Cannot parse smartctl output: {}"), std::make_format_args(message)));
+				fmt::format(fmt::runtime(_("Cannot parse smartctl output: {}")), message));
 	}
 
 	auto basic_property_repo = basic_parser->get_property_repository();
