@@ -1,8 +1,8 @@
-/**************************************************************************
- Copyright:
-      (C) 2008 - 2012  Alexander Shaduri <ashaduri 'at' gmail.com>
- License: See LICENSE_zlib.txt file
-***************************************************************************/
+/******************************************************************************
+License: Zlib
+Copyright:
+	(C) 2008 - 2021 Alexander Shaduri <ashaduri@gmail.com>
+******************************************************************************/
 /// \file
 /// \author Alexander Shaduri
 /// \ingroup libdebug
